#IF 0
  =============================================================================
  File NAME   : DynArrays.INC
  Author      : Eros Olmi
  Version     : 01.00.05
  Description : Dynamic Arrays
  Compiler    : PBWIN80
  Web site    : http://www.thinbasic.com
  Updates     : http://www.thinbasic.com/index.php?name=CmodsDownload&file=index&req=viewsdownload&sid=36&orderby=dateD
  =============================================================================
  COPYRIGHT AND PERMISSION NOTICE
  =============================================================================
  Copyright (c) 2003 - 2006, Eros Olmi, <eros.olmi@thinBasic.com>
   
  All rights reserved.
   
  Permission to use and copy this software for any purpose with or without fee
  is hereby granted, provided that the above copyright appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
  USE OR OTHER DEALINGS IN THE SOFTWARE.
  
  =============================================================================
  Description:
  -----------------------------------------------------------------------------
  Dynamically creates arrays at program runtime.
  Max dimensions: 3. Any numeric type, dynamic strings, UDT.
  All array information are stored in a structure and a pointer to array
  structure is returned. This allows easy inclusion of arrays inside UDT
  or any other data structure. Just a 32bit variable is needed.

  =============================================================================
  Edit History:       (Legend: +Add -Fix *Modify #Note)
  -----------------------------------------------------------------------------
  Date (YMD) Version  Comments
  -----------------------------------------------------------------------------
  2004.01.12 01.00.00 # First beta version
  2004.01.14          + New numeric types
  2004.01.15          * Changed all specific numeric functions used for Set/Get
                        an array item. Now all functions use REDIM..AT to
                        simplify and speedup operations.
                        Thanks to Michael Mattias, Chriss Boss for pointing out
                        that.
  2004.01.17 01.00.01 + Multi dimensions. Max %MaxDimensions
                      + LinearPos function convert from multi dims to
                        linear position
             01.00.02 * Array_GetItem_Str and Array_SetItem_Str: added
                        multidemension handling
                      + Basic bound checking. In case of out of bound, no error
                        but exit function. Numbers will return 0, strings
                        will return ""
                        This will anyhow change in  order to have more prof.
                        error handling
             01.00.03 + LinearPos changed in order to apply a generic formula
                        to calculate linear position of an array element
                        giving its dimension indexes
             01.00.04 * %MaxDimensions = 4 
  2005.01.29 01.00.05 * code completelly revised.
                        - Array_Dim now returns a pointer and not a string.
                          This allow very easy inclusion of arrays inside
                          any data structure
                        - speed increase
                        - 
  2005.02.25          * code revised again
                      + array_redim now works fine both for increase and
                        decrease number of items
  2006.01.25 01.00.06 * some more comments
                        
  =============================================================================
  To do:
  -----------------------------------------------------------------------------
  Item                Comment
  -----------------------------------------------------------------------------
  Error checking      Add error checking. Options: do at global level or at
                      single array level. Another way: global level but only
                      last error recorded like PB function ERRCLEAR
                      Suggestion: Chris Boss
  =============================================================================
#EndIf


  '----------------------------------------------------------------------------
  '----------------------------------------------------------------------------
  %MaxArrayIDLen            =  4&         'Max len of array ID, see $InternalID
  $InternalID               = "@@"      'More or less it is like a Watermark ...
  %MaxArrayUserIDLen        = 32&         'Max len of array User ID
  %MaxArrayNameLen          = 64&         'Max len of array name/description
  %MaxDimensions            =  4&         'Max number of dimensions inside an array
  '----------------------------------------------------------------------------

  '----------------------------------------------------------------------------
  ' Data type                      | Bytes | Done
  '----------------------------------------------------------------------------
  %ArrayType_Byte           =  1&  '   1      OK
  %ArrayType_Integer        =  2&  '   2      OK
  %ArrayType_Word           =  3&  '   2      OK
  %ArrayType_DWord          =  4&  '   4      OK
  %ArrayType_Long           =  5&  '   4      OK
  %ArrayType_Quad           =  6&  '   8      OK
  %ArrayType_Single         =  7&  '   4      OK
  %ArrayType_Double         =  8&  '   8      OK
  %ArrayType_Currency       =  9&  '   8      OK
  %ArrayType_Ext            = 10&  '  10      OK

  %ArrayType_AsciiZ         = 15&  '   4
  
  %ArrayType_IsNumber       = 20&  ' 
  %ArrayType_String         = 30&  '   4      OK
  %ArrayType_IsString       = 30&  '   4      OK

  %ArrayType_Variant        = 50&  '  16
  %ArrayType_UDT            = 60&  '   4
  '----------------------------------------------------------------------------

  '----------------------------------------------------------------------------
  ' Data type size          | Bytes
  '----------------------------------------------------------------------------
  %ArrayType_Byte_Size      =  1&
  %ArrayType_Integer_Size   =  2&
  %ArrayType_Word_Size      =  2&
  %ArrayType_DWord_Size     =  4&
  %ArrayType_Long_Size      =  4&
  %ArrayType_Quad_Size      =  8&
  %ArrayType_Single_Size    =  4&
  %ArrayType_Double_Size    =  8&
  %ArrayType_Currency_Size  =  8&
  %ArrayType_Ext_Size       = 10&
  %ArrayType_String_Size    =  4&  'Array of strings is in reality an array of pointers
  %ArrayType_AsciiZ_Size    =  4&  'Array of asciiz is in reality an array of pointers
  %ArrayType_Variant_Size   = 16&
  %ArrayType_UDT_Size       =  4&  'Array of UDT is in reality an array of pointers
  '----------------------------------------------------------------------------

  '----------------------------------------------------------------------------
  ' Sort type
  '----------------------------------------------------------------------------
  %Array_Sort_Ascending     =  1&
  %Array_Sort_Descending    =  2&
  

  '----------------------------------------------------------------------------
  ' Main structure used to identify the array. This structure
  ' is embedded as first part of the string array.
  '----------------------------------------------------------------------------
  TYPE sArrayDescriptor
    sID                 AS STRING * %MaxArrayIDLen      'Internally used to identify if...
                                                        '...pointer is a DynArray
    sUserID             AS STRING * %MaxArrayUserIDLen  'User defined ID
    sName               AS STRING * %MaxArrayNameLen    'Uder defined name/description

    lRefArrayPtr        AS LONG                         'An array can be a reference of another array
                                                        'In this case we have to use referenced array
    lRefArrayPos        AS LONG                         'An array can be a reference of another array
                                                        'This is the position inside the aray
                                                        
    lElementsArePtr     AS LONG                         '1=Elements are pointers   0=No
    lElementType        AS LONG                         'Type of elements stored
    lElementSize        AS LONG                         'Size of the single element
    lElementsCount      AS LONG                         'Number of elements stored
    lDimensions         AS LONG                         'Number of dimensions
    lDimLBound(1 TO %MaxDimensions)  AS LONG            'LBound for each dimension (default 1)
    lDimUBound(1 TO %MaxDimensions)  AS LONG            'UBound for each dimension                       
    pDataStorage        AS LONG                         'Pointer to effective data storage area

    '---From here you can additional personal data to store inside array info
    '---
    lLastUsedItem       AS LONG                         'Some internal functions update

    '---
    pPrevious           AS DWORD                        'Used in case you need a pointer to another Array
    pNext               AS DWORD                        'Used in case you need a pointer to another Array

  END TYPE
  

  
  '----------------------------------------------------------------------------
  ' Return bytes required to strore element type
  '----------------------------------------------------------------------------
  FUNCTION Array_ElementSize(BYVAL ElementType AS LONG) AS LONG
  
    SELECT CASE LONG ElementType
      CASE %ArrayType_Byte    :        FUNCTION = %ArrayType_Byte_Size
      CASE %ArrayType_Integer :        FUNCTION = %ArrayType_Integer_Size
      CASE %ArrayType_Word    :        FUNCTION = %ArrayType_Word_Size
      CASE %ArrayType_DWord   :        FUNCTION = %ArrayType_DWord_Size
      CASE %ArrayType_Long    :        FUNCTION = %ArrayType_Long_Size
      CASE %ArrayType_Quad    :        FUNCTION = %ArrayType_Quad_Size 
      
      CASE %ArrayType_Single  :        FUNCTION = %ArrayType_Single_Size
      CASE %ArrayType_Double  :        FUNCTION = %ArrayType_Double_Size
      CASE %ArrayType_Currency:        FUNCTION = %ArrayType_Currency_Size

      CASE %ArrayType_Ext     :        FUNCTION = %ArrayType_Ext_Size

      CASE %ArrayType_AsciiZ  :        FUNCTION = %ArrayType_AsciiZ_Size
      CASE %ArrayType_String  :        FUNCTION = %ArrayType_String_Size
      CASE %ArrayType_UDT     :        FUNCTION = %ArrayType_UDT_Size
      CASE %ArrayType_Variant :        FUNCTION = %ArrayType_Variant_Size
      CASE ELSE
        '-----------> ERROR
    END SELECT
  
  END FUNCTION

  '----------------------------------------------------------------------------
  ' Get the pointer to data storage
  '----------------------------------------------------------------------------
    FUNCTION Array_GetDataPtr(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.pDataStorage
    END FUNCTION  
  
  '----------------------------------------------------------------------------
  ' Get the array total size. In case of strings or UDT this size does not
  ' include real data but only data needed to store the array
  '----------------------------------------------------------------------------
    FUNCTION Array_GetSize(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = da_MemSize(pArray) + da_MemSize(@pArray.pDataStorage)
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get/Set the array descriptor record
  '----------------------------------------------------------------------------
    SUB Array_GetArrayDescriptor(BYVAL pArray AS LONG, AD AS sArrayDescriptor)
      POKE$ VARPTR(AD), PEEK$(pArray, SIZEOF(sArrayDescriptor))
    END SUB  
    '---
    SUB Array_SetArrayDescriptor(BYVAL pArray AS sArrayDescriptor PTR, AD AS sArrayDescriptor)
      POKE$ pArray, PEEK$(VARPTR(AD), SIZEOF(sArrayDescriptor))   
    END SUB  

  '----------------------------------------------------------------------------
  ' Get the array internal ID
  '----------------------------------------------------------------------------
    FUNCTION Array_GetID(BYVAL pArray AS sArrayDescriptor PTR) AS STRING
      FUNCTION = @pArray.sID
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the array user defined ID
  '----------------------------------------------------------------------------
    FUNCTION Array_GetUserID(BYVAL pArray AS sArrayDescriptor PTR) AS STRING
      FUNCTION = TRIM$(@pArray.sUserID)
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the array name
  '----------------------------------------------------------------------------
    FUNCTION Array_GetName(BYVAL pArray AS sArrayDescriptor PTR) AS STRING
      FUNCTION = TRIM$(@pArray.sName)
    END FUNCTION  
  
  '----------------------------------------------------------------------------
  ' Get if elements are pointers 1=YES, 0=NO
  '----------------------------------------------------------------------------
    FUNCTION Array_ElementsArePtr(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.lElementsArePtr
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the array elements type
  '----------------------------------------------------------------------------
    FUNCTION Array_GetElementType(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.lElementType   
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the array element size
  '----------------------------------------------------------------------------
    FUNCTION Array_GetElementSize(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.lElementSize   
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the number of dimensions int the array
  '----------------------------------------------------------------------------
    FUNCTION Array_GetDimensions(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.lDimensions   
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the Upper Bound of a dimension
  '----------------------------------------------------------------------------
    FUNCTION Array_GetUbound(BYVAL pArray AS sArrayDescriptor PTR, lDimension AS LONG) AS LONG
      IF lDimension > 0& AND lDimension <= @pArray.lDimensions THEN
        FUNCTION = @pArray.lDimUBound(lDimension)
      END IF   
    END FUNCTION  

  '----------------------------------------------------------------------------
  ' Get the total number of elements in the array
  '----------------------------------------------------------------------------
    FUNCTION Array_GetElementsCount(BYVAL pArray AS sArrayDescriptor PTR) AS LONG
      FUNCTION = @pArray.lElementsCount   
    END FUNCTION  
  
  '----------------------------------------------------------------------------
  ' Calculate the position of the element inside memory allocated giving
  ' dimensions parameters. Linearize multidimension array
  ' Reference: http://webster.cs.ucr.edu/Page_AoALinux/HTML/Arraysa2.html
  '            http://www.uscms.org/s&c/proposals/itr2003/docs/UofM-Metadata-1.pdf
  '----------------------------------------------------------------------------
  FUNCTION Array_LinearPos(BYVAL pArray AS sArrayDescriptor PTR, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS LONG

    '---Try to speedup operations
    IF UBound2 = 0 THEN
      FUNCTION = UBound1
      EXIT FUNCTION
    END IF

    'LOCAL PosM   AS LONG 
    'LOCAL PosD   AS LONG
    'LOCAL CountM AS LONG
    'LOCAL CountD AS LONG 
    LOCAL PassedDimensions AS LONG

    '---
    'Here there must be a better way to load dimensions passed as parameters
    'For now this is the way    
    '---
    PassedDimensions = 1& + iif&(UBound2 = 0&, 0&, 1&) + iif&(UBound3 = 0&, 0&, 1&) + iif&(UBound4 = 0&, 0&, 1&)
    
    IF PassedDimensions = 1& THEN
      FUNCTION = UBound1
      EXIT FUNCTION
    END IF
 
    IF  UBound1 < @pArray.lDimLBound(1&) OR UBound1 > @pArray.lDimUBound(1&) THEN
       FUNCTION = -1&
       EXIT FUNCTION
    END IF
    IF  UBound2 < @pArray.lDimLBound(2&) OR UBound2 > @pArray.lDimUBound(2&) THEN
       FUNCTION = -2&
       EXIT FUNCTION
    END IF
    IF  UBound3 < @pArray.lDimLBound(3&) OR UBound3 > @pArray.lDimUBound(3&) THEN
       FUNCTION = -3&
       EXIT FUNCTION
    END IF
    IF  UBound4 < @pArray.lDimLBound(4&) OR UBound4 > @pArray.lDimUBound(4&) THEN
       FUNCTION = -4&
       EXIT FUNCTION
    END IF

    
    '---
    'Assume that the cardinality OF dimension i OF R is di, FOR 1 <= i <= n.
    'The position IN the linearized ARRAY OF the multidimensional ARRAY element WITH
    'values (x1, x2, ..., xn), is 
    ' 
    'LINEAR(x1, x2, ..., xn) = x1x2x3...xn + x2x3...xn +...+ xn-1xn +xn
    '                        =(...(x1x2+x2)x3+...)xn-2+xn-2)xn-1+xn-1)xn+xn
    'Examples:
    '2 dimensions: 
    '   LINEAR(x1, x2) = x1x2 + x2
    '3 dimensions: 
    '   LINEAR(x1, x2, x3) = x1x2x3 + x2x3 + x3
    '4 dimensions: 
    '   LINEAR(x1, x2, x3, x4) = x1x2x3x4 + x2x3x4 + x3x4 + x4
    '---


    SELECT CASE LONG PassedDimensions
      CASE 1
        FUNCTION =  _
                    UBound1
      CASE 2
        FUNCTION =  _
                    @pArray.lDimUBound(1&) * (UBound2 - 1) + UBound1
      CASE 3
        FUNCTION =  _
                    (((UBound3-1) * @pArray.lDimUBound(2&) + (UBound2-1))*@pArray.lDimUBound(1&)+UBound1)

      CASE ELSE
        FUNCTION = 0
    END SELECT    

  END FUNCTION  
  
  '----------------------------------------------------------------------------
  ' Create a new array
  '----------------------------------------------------------------------------
  FUNCTION Array_Dim(BYVAL UserID AS STRING, BYVAL ArrayName AS STRING, BYVAL ElementType AS LONG, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS LONG
    LOCAL AD        AS sArrayDescriptor
    LOCAL pArray    AS sArrayDescriptor PTR
    
    AD.sID             = $InternalID
    AD.sUserID         = UserID
    AD.sName           = ArrayName
    AD.lElementType    = ElementType
    AD.lElementsArePtr = IIF(ElementType < %ArrayType_IsNumber, 0&, 1&)
    AD.lElementSize    = Array_ElementSize(ElementType)

    UBound1 = iif&(UBound1 = 0&, 1&, UBound1)
    AD.lElementsCount  = _
                          IIF&(UBound1 = 0&, 1&, UBound1) * _
                          IIF&(UBound2 = 0&, 1&, UBound2) * _
                          IIF&(UBound3 = 0&, 1&, UBound3) * _
                          IIF&(UBound4 = 0&, 1&, UBound4)
    AD.lDimensions     = _
                          1& + _
                          IIF&(UBound2 = 0&, 0&, 1&) + _
                          IIF&(UBound3 = 0&, 0&, 1&) + _
                          IIF&(UBound4 = 0&, 0&, 1&)

    AD.lDimLBound(1&)   = 1&
    AD.lDimLBound(2&)   = IIF&(UBound2 = 0&, 0&, 1&)
    AD.lDimLBound(3&)   = IIF&(UBound3 = 0&, 0&, 1&)
    AD.lDimLBound(4&)   = IIF&(UBound4 = 0&, 0&, 1&)
    
    AD.lDimUBound(1&)   = UBound1
    AD.lDimUBound(2&)   = UBound2
    AD.lDimUBound(3&)   = UBound3
    AD.lDimUBound(4&)   = UBound4
    
    AD.pDataStorage = da_MemAlloc( AD.lElementsCount * AD.lElementSize )
                      
    pArray = da_MemAlloc( SIZEOF(sArrayDescriptor) )
                      
    
    Array_SetArrayDescriptor pArray, AD 

    FUNCTION = pArray
    
  END FUNCTION  

  FUNCTION Array_Redim_ClearData(BYVAL pArray AS sArrayDescriptor PTR, lPreserve AS LONG, lCurElementsCount AS LONG) AS LONG
    LOCAL Count  AS LONG
    LOCAL lStart AS LONG
    LOCAL lEnd   AS LONG

    lStart = IIF&(lPreserve = %TRUE, lCurElementsCount + 1&, 1&)
    lEnd   = lCurElementsCount

    SELECT CASE LONG @pArray.lElementType

      CASE %ArrayType_String, %ArrayType_UDT, %ArrayType_AsciiZ
        REDIM dummyDoubleStrPtr(1& TO lCurElementsCount) AS DWORD  AT @pArray.pDataStorage
        REDIM dummyString(1& TO lCurElementsCount) AS STRING AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          IF dummyDoubleStrPtr(Count) THEN
            RESET dummyString(Count)
          END IF
        NEXT

      CASE %ArrayType_Byte
        REDIM dummyByte(1& TO lCurElementsCount) AS BYTE AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyByte(Count)
        NEXT
      CASE %ArrayType_Integer
        REDIM dummyInteger(1& TO lCurElementsCount) AS INTEGER AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyInteger(Count)
        NEXT
      CASE %ArrayType_Long
        REDIM dummyLong(1& TO lCurElementsCount) AS LONG AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyLong(Count)
        NEXT
      CASE %ArrayType_Quad
        REDIM dummyQuad(1& TO lCurElementsCount) AS QUAD AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyQuad(Count)
        NEXT
      CASE %ArrayType_Single
        REDIM dummySingle(1& TO lCurElementsCount) AS SINGLE AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummySingle(Count)
        NEXT
      CASE %ArrayType_Double
        REDIM dummyDouble(1& TO lCurElementsCount) AS DOUBLE AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyDouble(Count)
        NEXT
      CASE %ArrayType_Currency
        REDIM dummyCurrency(1& TO lCurElementsCount) AS CURRENCY AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummycurrency(Count)
        NEXT
      CASE %ArrayType_Ext
        REDIM dummyExt(1& TO lCurElementsCount) AS EXT  AT @pArray.pDataStorage
        FOR Count = lStart TO lEnd
          RESET dummyExt(Count)
        NEXT
        
    END SELECT
        
  END FUNCTION

  '----------------------------------------------------------------------------
  ' Redim and array
  '----------------------------------------------------------------------------
  FUNCTION Array_Redim(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lPreserve AS LONG, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS LONG
    LOCAL lNewElementsCount AS LONG
    LOCAL lCurElementsCount AS LONG

    IF pArray <= 0& THEN EXIT FUNCTION
    
    UBound1 = IIF&(UBound1 = 0&, 1&, UBound1)
    
    lCurElementsCount = @pArray.lElementsCount
    
    lNewElementsCount = _
                        UBound1 * _
                        IIF&(UBound2 = 0&, 1&, UBound2) * _
                        IIF&(UBound3 = 0&, 1&, UBound3) * _
                        IIF&(UBound4 = 0&, 1&, UBound4)


    '---If same element number, exit
    IF lNewElementsCount = lCurElementsCount THEN
      EXIT FUNCTION
                    
    ELSE
    
      @pArray.lDimLBound(1)   = 1&
      @pArray.lDimLBound(2)   = IIF&(UBound2 = 0&, 0&, 1&)
      @pArray.lDimLBound(3)   = IIF&(UBound3 = 0&, 0&, 1&)
      @pArray.lDimLBound(4)   = IIF&(UBound4 = 0&, 0&, 1&)

      @pArray.lDimUBound(1)   = UBound1
      @pArray.lDimUBound(2)   = UBound2
      @pArray.lDimUBound(3)   = UBound3
      @pArray.lDimUBound(4)   = UBound4

      '---If to add more elements, just add more space to the main string array
      IF lNewElementsCount > lCurElementsCount THEN 

        @pArray.lElementsCount = lNewElementsCount

        Array_Redim_ClearData(pArray, lPreserve, lCurElementsCount)

        @pArray.pDataStorage = da_MemReAlloc( _
                                  @pArray.pDataStorage, _
                                  lNewElementsCount * @pArray.lElementSize _
                                )
      ELSE

        Array_Redim_ClearData(pArray, lPreserve, lCurElementsCount)
        
        '---Now we are ready to redim
        @pArray.lElementsCount = lNewElementsCount
        @pArray.pDataStorage = da_MemReAlloc( _
                                  @pArray.pDataStorage, _
                                  lNewElementsCount * @pArray.lElementSize _
                                )
        
        
      END IF
      
      '---Ok, now the array should be ready.
      '---Check if preserve values
      IF lPreserve = %TRUE THEN
      ELSE  
      END IF

    END IF
    
  END FUNCTION  

  '----------------------------------------------------------------------------
  ' Delete a new array
  '----------------------------------------------------------------------------
  SUB Array_Free(BYVAL pArray AS sArrayDescriptor PTR)

    IF pArray <= 0& THEN EXIT SUB
    
    '---
    'In case of array of:
    ' - strings
    'we must first deallocate memory
    '---
    SELECT CASE LONG @pArray.lElementType
      CASE %ArrayType_String, %ArrayType_UDT, %ArrayType_AsciiZ

        LOCAL lItems AS LONG
        LOCAL Count  AS LONG
        lItems = @pArray.lElementsCount

        'REDIM dummyDouble(1& TO lItems) AS DWORD  AT @pArray.pDataStorage
        REDIM dummyString(1& TO lItems) AS STRING AT @pArray.pDataStorage

        'FOR Count = 1& TO lItems
        '  IF dummyDouble(Count) THEN
        '    RESET dummyString(Count)
        '    'RESET dummyString()
        '  END IF
        'NEXT
        ERASE dummyString()
    END SELECT
    
    '---
    'In case there are other associated arrays, we need to delete them.
    'This is done calling Array_Erase recursively
    '---
    IF @pArray.pPrevious > 0& THEN Array_Erase @pArray.pPrevious
    IF @pArray.pNext     > 0& THEN Array_Erase @pArray.pNext

    '---
    'Now we can free array data
    '---
    da_MemFree(@pArray.pDataStorage)   
    
    '---
    'Now we can free array header
    '---
    da_MemFree(pArray)   
    
  END SUB

  '----------------------------------------------------------------------------
  ' Delete a new array (alias of Array_Free)
  '----------------------------------------------------------------------------
  SUB Array_Erase(BYVAL pArray AS sArrayDescriptor PTR)
    Array_Free(pArray)
  END SUB
  


  '----------------------------------------------------------------------------
  '----------------------------------------------------------------------------
  '----------------------------------------------------------------------------
  
  
  '----------------------------------------------------------------------------
  ' BYTE: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Byte(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS BYTE)
      POKE BYTE, (@pArray.pDataStorage + (%ArrayType_Byte_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Byte(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS BYTE
      FUNCTION = PEEK(BYTE, (@pArray.pDataStorage + (%ArrayType_Byte_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' INTEGER: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Integer(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS INTEGER)
      POKE INTEGER, (@pArray.pDataStorage + (%ArrayType_Integer_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Integer(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS INTEGER
      FUNCTION = PEEK(INTEGER, (@pArray.pDataStorage + (%ArrayType_Integer_Size * (lItem - 1&))))
    END FUNCTION 
    
  '----------------------------------------------------------------------------
  ' WORD: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Word(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG,  BYVAL lItemValue AS WORD)
      POKE WORD, (@pArray.pDataStorage + (%ArrayType_Word_Size * (lItem - 1&))), lItemValue
    END SUB
    '---                                                                               
    FUNCTION Array_GetItem_Word(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS WORD
      FUNCTION = PEEK(WORD, (@pArray.pDataStorage + (%ArrayType_Word_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' DWORD: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_DWord(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG,  BYVAL lItemValue AS DWORD)
      POKE DWORD, (@pArray.pDataStorage + (%ArrayType_DWord_Size * (lItem - 1&))), lItemValue
    END SUB
    '---                                                                               
    FUNCTION Array_GetItem_DWord(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS DWORD
      FUNCTION = PEEK(DWORD, (@pArray.pDataStorage + (%ArrayType_DWord_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' LONG: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Long(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG,  BYVAL lItemValue AS LONG)
      POKE LONG, (@pArray.pDataStorage + (%ArrayType_Long_Size * (lItem - 1&))), lItemValue
    END SUB
    '---                                                                               
    FUNCTION Array_GetItem_Long(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS LONG
      FUNCTION = PEEK(LONG, (@pArray.pDataStorage + (%ArrayType_Long_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' QUAD: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Quad(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS QUAD)
      POKE QUAD, (@pArray.pDataStorage + (%ArrayType_Quad_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Quad(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS QUAD
      FUNCTION = PEEK(QUAD, (@pArray.pDataStorage + (%ArrayType_Quad_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' SINGLE: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Single(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS SINGLE)
      POKE SINGLE, (@pArray.pDataStorage + (%ArrayType_Single_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Single(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS SINGLE
      FUNCTION = PEEK(SINGLE, (@pArray.pDataStorage + (%ArrayType_Single_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' DOUBLE: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Double(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS DOUBLE)
      POKE DOUBLE, (@pArray.pDataStorage + (%ArrayType_Double_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Double(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS DOUBLE
      FUNCTION = PEEK(DOUBLE, (@pArray.pDataStorage + (%ArrayType_Double_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' CURRENCY: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Currency(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS CURRENCY)
      POKE CUR, (@pArray.pDataStorage + (%ArrayType_Currency_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Currency(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS CURRENCY
      FUNCTION = PEEK(CUR, (@pArray.pDataStorage + (%ArrayType_Currency_Size * (lItem - 1&))))
    END FUNCTION

  '----------------------------------------------------------------------------
  ' EXT: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Ext(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG, BYVAL lItemValue AS EXT)
      POKE EXT, (@pArray.pDataStorage + (%ArrayType_Ext_Size * (lItem - 1&))), lItemValue
    END SUB
    '---
    FUNCTION Array_GetItem_Ext(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lItem AS LONG) AS EXT
      FUNCTION = PEEK(EXT, (@pArray.pDataStorage + (%ArrayType_Ext_Size * (lItem - 1&))))
    END FUNCTION


  '----------------------------------------------------------------------------
  ' Get VARPTR/STRPTR of an array item
  '----------------------------------------------------------------------------
    FUNCTION Array_VARPTR(BYVAL pArray AS sArrayDescriptor PTR, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS DWORD
      LOCAL lItem AS LONG
      LOCAL pData AS DWORD

      IF pArray = 0& THEN EXIT FUNCTION
      
      '---If second dimension is 0 it means only one dimension, so UBound1 is the index
      IF UBound2 = 0& THEN
        lItem = UBound1
      ELSE
        lItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
      END IF
      
      pData = (@pArray.pDataStorage + (@pArray.lElementSize * (lItem - 1&)))

      FUNCTION = pData
    END FUNCTION
    
    FUNCTION Array_STRPTR(BYVAL pArray AS sArrayDescriptor PTR, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS DWORD
      LOCAL lItem AS LONG
      LOCAL pData AS DWORD

      IF pArray = 0& THEN EXIT FUNCTION
      
      '---If second dimension is 0 it means only one dimension, so UBound1 is the index
      IF UBound2 = 0& THEN
        lItem = UBound1
      ELSE
        lItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
      END IF 
      
      pData = (@pArray.pDataStorage + (%ArrayType_String_Size * (lItem - 1&)))
      REDIM dummyString(1& TO 1&) AS STRING AT pData
      FUNCTION = STRPTR(dummyString(1&))

    END FUNCTION

  '----------------------------------------------------------------------------
  ' Generic Number: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Num(BYVAL pArray AS sArrayDescriptor PTR, BYVAL ItemValue AS EXT, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG)
      LOCAL LinearPosByRef  AS LONG
      
      'IF pArray = 0& THEN EXIT SUB  
      
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        LinearPosByRef  = @pArray.lRefArrayPos
        pArray = @pArray.lRefArrayPtr
      WEND
      
      IF LinearPosByRef  = 0& THEN
        '---If second dimension is 0 it means only one dimension, so UBound1 is the index
        IF UBound2 = 0& THEN
          @pArray.lLastUsedItem = uBound1
        ELSE
          @pArray.lLastUsedItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
        END IF
        'IF lItem <= 0& THEN EXIT SUB
      ELSE
        @pArray.lLastUsedItem = LinearPosByRef
      END IF

      SELECT CASE LONG @pArray.lElementType
  
        
        CASE %ArrayType_Long      :   Array_SetItem_Long      pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Byte      :   Array_SetItem_Byte      pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Integer   :   Array_SetItem_Integer   pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Word      :   Array_SetItem_Word      pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_DWord     :   Array_SetItem_DWOrd     pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Quad      :   Array_SetItem_Quad      pArray, @pArray.lLastUsedItem, ItemValue
  
        CASE %ArrayType_Single    :   Array_SetItem_Single    pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Double    :   Array_SetItem_Double    pArray, @pArray.lLastUsedItem, ItemValue
        CASE %ArrayType_Ext       :   Array_SetItem_Ext       pArray, @pArray.lLastUsedItem, ItemValue 
        CASE %ArrayType_Currency  :   Array_SetItem_Currency  pArray, @pArray.lLastUsedItem, ItemValue
  
        CASE ELSE
          'ERROR
      END SELECT
      
    END SUB
    '---
    FUNCTION Array_GetItem_Num(BYVAL pArray AS sArrayDescriptor PTR, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS EXT
      LOCAL LinearPosByRef  AS LONG
      
      'IF pArray = 0& THEN EXIT FUNCTION
      '---Here test IF ARRAY POINTER refer TO another ARRAY. IF yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        LinearPosByRef  = @pArray.lRefArrayPos
        pArray          = @pArray.lRefArrayPtr
      WEND
      
      IF LinearPosByRef  = 0& THEN
        '---If second dimension is 0 it means only one dimension, so UBound1 is the index
        IF UBound2 = 0& THEN
          @pArray.lLastUsedItem = uBound1
        ELSE
          @pArray.lLastUsedItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
        END IF
        'IF lItem <= 0& THEN EXIT FUNCTION
      ELSE
        @pArray.lLastUsedItem = LinearPosByRef
      END IF
            
      SELECT CASE LONG @pArray.lElementType
               
        CASE %ArrayType_Long      :   FUNCTION = Array_GetItem_Long     (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Byte      :   FUNCTION = Array_GetItem_Byte     (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Integer   :   FUNCTION = Array_GetItem_Integer  (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Word      :   FUNCTION = Array_GetItem_Word     (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_DWord     :   FUNCTION = Array_GetItem_DWord    (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Quad      :   FUNCTION = Array_GetItem_Quad     (pArray, @pArray.lLastUsedItem)
         
        CASE %ArrayType_Single    :   FUNCTION = Array_GetItem_Single   (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Double    :   FUNCTION = Array_GetItem_Double   (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Ext       :   FUNCTION = Array_GetItem_Ext      (pArray, @pArray.lLastUsedItem)
        CASE %ArrayType_Currency  :   FUNCTION = Array_GetItem_Currency (pArray, @pArray.lLastUsedItem)
        
        CASE ELSE
          'ERROR
      END SELECT
      
    END FUNCTION

  '----------------------------------------------------------------------------
  ' Fill the whole numeric array with a specific numeric value
  '----------------------------------------------------------------------------
    SUB Array_Fill_Num(BYVAL pArray AS sArrayDescriptor PTR, BYVAL ItemValue AS EXT)
      LOCAL Count AS LONG
      
      'IF pArray = 0& THEN EXIT SUB  
      
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        pArray = @pArray.lRefArrayPtr
      WEND

      FOR Count = 1 TO @pArray.lElementsCount

        SELECT CASE LONG @pArray.lElementType
          CASE %ArrayType_Long      :   Array_SetItem_Long      pArray, Count, ItemValue
          CASE %ArrayType_Byte      :   Array_SetItem_Byte      pArray, Count, ItemValue
          CASE %ArrayType_Integer   :   Array_SetItem_Integer   pArray, Count, ItemValue
          CASE %ArrayType_Word      :   Array_SetItem_Word      pArray, Count, ItemValue
          CASE %ArrayType_DWord     :   Array_SetItem_DWOrd     pArray, Count, ItemValue
          CASE %ArrayType_Quad      :   Array_SetItem_Quad      pArray, Count, ItemValue
    
          CASE %ArrayType_Single    :   Array_SetItem_Single    pArray, Count, ItemValue
          CASE %ArrayType_Double    :   Array_SetItem_Double    pArray, Count, ItemValue
          CASE %ArrayType_Ext       :   Array_SetItem_Ext       pArray, Count, ItemValue 
          CASE %ArrayType_Currency  :   Array_SetItem_Currency  pArray, Count, ItemValue
    
          CASE ELSE
            'ERROR
        END SELECT
      NEXT      
    END SUB

  '----------------------------------------------------------------------------
  ' STRING: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_Str(BYVAL pArray AS sArrayDescriptor PTR, sValue AS STRING, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) 
      LOCAL LinearPosByRef AS LONG
      
      'IF pArray = 0& THEN EXIT SUB
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        LinearPosByRef = @pArray.lRefArrayPos
        pArray         = @pArray.lRefArrayPtr
      WEND

      IF LinearPosByRef = 0& THEN
        '---If second dimension is 0 it means only one dimension, so UBound1 is the index
        IF UBound2 = 0& THEN
          @pArray.lLastUsedItem = UBound1
        ELSE
          @pArray.lLastUsedItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
        END IF
        'IF lItem <= 0& THEN EXIT SUB  
      ELSE
          @pArray.lLastUsedItem = LinearPosByRef
      END IF
    
      REDIM dummyString(1& TO @pArray.lLastUsedItem) AS STRING AT @pArray.pDataStorage

      'SELECT CASE LONG @pArray.lElementType
      '  CASE %ArrayType_String
          dummyString(@pArray.lLastUsedItem) = sValue
      '  CASE %ArrayType_AsciiZ
      '    dummyString(@pArray.lLastUsedItem) = sValue & CHR$(0)
      'END SELECT

    END SUB          
    '---
    FUNCTION Array_GetItem_Str(BYVAL pArray AS sArrayDescriptor PTR,  BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS STRING
      LOCAL LinearPosByRef AS LONG
      
      'IF pArray = 0& THEN EXIT FUNCTION
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        LinearPosByRef = @pArray.lRefArrayPos
        pArray         = @pArray.lRefArrayPtr
      WEND

      IF LinearPosByRef = 0& THEN
        '---If second dimension is 0 it means only one dimension, so UBound1 is the index
        IF UBound2 = 0& THEN
          @pArray.lLastUsedItem = UBound1
        ELSE
          @pArray.lLastUsedItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
        END IF
        'IF lItem <= 0& THEN EXIT FUNCTION  
      ELSE
          @pArray.lLastUsedItem = LinearPosByRef
      END IF
          
      REDIM dummyString(1& TO @pArray.lLastUsedItem) AS STRING AT @pArray.pDataStorage
  
      'SELECT CASE LONG @pArray.lElementType
      '  CASE %ArrayType_String
          FUNCTION = dummyString(@pArray.lLastUsedItem)
      '  CASE %ArrayType_AsciiZ
      '    FUNCTION = LEFT$(dummyString(@pArray.lLastUsedItem), -1)
      'END SELECT
      
    END FUNCTION
    '---
    SUB Array_Fill_Str(BYVAL pArray AS sArrayDescriptor PTR, sValue AS STRING)
      LOCAL Count AS LONG
      
      'IF pArray = 0& THEN EXIT SUB
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        pArray = @pArray.lRefArrayPtr
      WEND
   
      REDIM dummyString(1& TO @pArray.lElementsCount) AS STRING AT @pArray.pDataStorage

      FOR Count = 1 TO @pArray.lElementsCount

        SELECT CASE LONG @pArray.lElementType
          CASE %ArrayType_String
            dummyString(Count) = sValue
          CASE %ArrayType_AsciiZ
            dummyString(Count) = sValue & CHR$(0)
        END SELECT

      NEXT
    END SUB          

 
  '----------------------------------------------------------------------------
  ' UDT: Set/Get an array item value
  '----------------------------------------------------------------------------
    SUB Array_SetItem_UDT(BYVAL pArray AS sArrayDescriptor PTR, BYVAL pValue AS DWORD, BYVAL pSize AS LONG, BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG)
      LOCAL lItem AS LONG
      
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        pArray = @pArray.lRefArrayPtr
      WEND

      IF  pArray <= 0& OR _
          pValue <= 0& OR _
          pSize  <= 0& THEN EXIT SUB

      '---If second dimension is 0 it means only one dimension, so UBound1 is the index
      IF UBound2 = 0& THEN
        lItem = UBound1
      ELSE
        lItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
      END IF
      'IF lItem <= 0& THEN EXIT SUB  
    
      REDIM dummyString(1& TO lItem) AS STRING AT @pArray.pDataStorage
      dummyString(lItem) = PEEK$(pValue, pSize)
    END SUB          
    '---
    FUNCTION Array_GetItem_UDT(BYVAL pArray AS sArrayDescriptor PTR,  BYVAL UBound1 AS LONG, OPTIONAL BYVAL UBound2 AS LONG, BYVAL UBound3 AS LONG, BYVAL UBound4 AS LONG) AS DWORD
      LOCAL lItem AS LONG
      
      'IF pArray = 0& THEN EXIT FUNCTION
      '---Here test if array pointer refer to another array. If yes, use it
      WHILE @pArray.lRefArrayPtr <> 0&
        pArray = @pArray.lRefArrayPtr
      WEND

      '---If second dimension is 0 it means only one dimension, so UBound1 is the index
      IF UBound2 = 0& THEN
        lItem = UBound1
      ELSE
        lItem = Array_LinearPos(pArray, UBound1, UBound2, UBound3, UBound4)
      END IF
      'IF lItem <= 0& THEN EXIT FUNCTION  
    
      REDIM dummyString(1& TO lItem) AS STRING AT @pArray.pDataStorage
  
      FUNCTION = STRPTR(dummyString(lItem))
      
    END FUNCTION


  '----------------------------------------------------------------------------
  ' Array sorting
  '----------------------------------------------------------------------------
  ' Parameters:
  '   pArray      : array pointer
  '   lIndex      : point where to start sorting
  '   lCount      : number of elements to sort
  '   lOrder      : sort direction. Use predefined equates please.
  '----------------------------------------------------------------------------
  SUB Array_Sort(BYVAL pArray AS sArrayDescriptor PTR, BYVAL lIndex AS QUAD, BYVAL lCount AS QUAD, OPTIONAL BYVAL lOrder AS LONG)
    LOCAL lItems AS LONG

    IF pArray <= 0& THEN EXIT SUB
    
    '---Here test if array pointer refer to another array. If yes, use it
    WHILE @pArray.lRefArrayPtr <> 0&
      pArray = @pArray.lRefArrayPtr
    WEND
    
    lItems = @pArray.lElementsCount
    
    SELECT CASE LONG @pArray.lElementType
      
      '---Strings and UDT
      CASE %ArrayType_String, %ArrayType_UDT, %ArrayType_AsciiZ
        REDIM dummyString(1& TO lItems) AS STRING AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyString(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyString(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyString(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyString(lIndex), DESCEND
          END SELECT
        END IF

      CASE %ArrayType_Byte
        REDIM dummyByte(1& TO lItems) AS BYTE AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyByte(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyByte(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyByte(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyByte(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Integer
        REDIM dummyInteger(1& TO lItems) AS INTEGER AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyInteger(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyInteger(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyInteger(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyInteger(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Long
        REDIM dummyLong(1& TO lItems) AS LONG AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyLong(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyLong(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyLong(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyLong(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Quad
        REDIM dummyQuad(1& TO lItems) AS QUAD AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyQuad(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyQuad(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyQuad(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyQuad(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Single
        REDIM dummySingle(1& TO lItems) AS SINGLE AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummySingle(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummySingle(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummySingle(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummySingle(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Double
        REDIM dummyDouble(1& TO lItems) AS DOUBLE AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyDouble(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyDouble(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyDouble(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyDouble(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Currency
        REDIM dummyCurrency(1& TO lItems) AS CURRENCY AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyCurrency(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyCurrency(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyCurrency(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyCurrency(lIndex), DESCEND
          END SELECT
        END IF
      
      CASE %ArrayType_Ext
        REDIM dummyExt(1& TO lItems) AS EXT AT @pArray.pDataStorage
        IF lCount > 0 THEN
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyExt(lIndex) FOR lCount, ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyExt(lIndex) FOR lCount, DESCEND
          END SELECT
        ELSE  
          lIndex = 1
          SELECT CASE lOrder
            CASE 0, %Array_Sort_Ascending
              ARRAY SORT dummyExt(lIndex), ASCEND 
            CASE %Array_Sort_Descending             
              ARRAY SORT dummyExt(lIndex), DESCEND
          END SELECT
        END IF
        
    END SELECT
    
  END SUB
  