#IF NOT %DEF(%WINAPI)

    %MINCHAR  = &H80?
    %MAXCHAR  = &H7F?
    %MINSHORT = &H8000%
    %MAXSHORT = &H7FFF%
    %MINLONG  = &H80000000&
    %MAXBYTE  = &HFF?
    %MAXWORD  = &HFFFF??
    %MAXDWORD = &HFFFFFFFF???
    
    %HEAP_NO_SERIALIZE             = &H00000001
    %HEAP_GROWABLE                 = &H00000002
    %HEAP_GENERATE_EXCEPTIONS      = &H00000004
    %HEAP_ZERO_MEMORY              = &H00000008
    %HEAP_REALLOC_IN_PLACE_ONLY    = &H00000010
    %HEAP_TAIL_CHECKING_ENABLED    = &H00000020
    %HEAP_FREE_CHECKING_ENABLED    = &H00000040
    %HEAP_DISABLE_COALESCE_ON_FREE = &H00000080
    %HEAP_CREATE_ALIGN_16          = &H00010000
    %HEAP_CREATE_ENABLE_TRACING    = &H00020000
    %HEAP_MAXIMUM_TAG              = &H00000FFF
    %HEAP_PSEUDO_TAG_FLAG          = &H00008000
    %HEAP_TAG_SHIFT                = 18

  DECLARE FUNCTION  HeapAlloc         LIB "KERNEL32.DLL" ALIAS "HeapAlloc"        (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD, BYVAL dwBytes AS DWORD) AS DWORD
  DECLARE FUNCTION  HeapCompact       LIB "KERNEL32.DLL" ALIAS "HeapCompact"      (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD) AS DWORD
  DECLARE FUNCTION  HeapCreate        LIB "KERNEL32.DLL" ALIAS "HeapCreate"       (BYVAL flOptions AS DWORD, BYVAL dwInitialSize AS DWORD, BYVAL dwMaximumSize AS DWORD) AS DWORD
  DECLARE FUNCTION  HeapDestroy       LIB "KERNEL32.DLL" ALIAS "HeapDestroy"      (BYVAL hHeap AS DWORD) AS LONG
  DECLARE FUNCTION  HeapFree          LIB "KERNEL32.DLL" ALIAS "HeapFree"         (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD, BYVAL lpMem AS DWORD) AS LONG
  DECLARE FUNCTION  HeapLock          LIB "KERNEL32.DLL" ALIAS "HeapLock"         (BYVAL hHeap AS DWORD) AS LONG
  DECLARE FUNCTION  HeapReAlloc       LIB "KERNEL32.DLL" ALIAS "HeapReAlloc"      (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD, BYVAL lpMem AS DWORD, BYVAL dwBytes AS DWORD) AS DWORD
  DECLARE FUNCTION  HeapSize          LIB "KERNEL32.DLL" ALIAS "HeapSize"         (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD, BYVAL lpMem AS DWORD) AS DWORD
  DECLARE FUNCTION  HeapUnlock        LIB "KERNEL32.DLL" ALIAS "HeapUnlock"       (BYVAL hHeap AS DWORD) AS LONG
  DECLARE FUNCTION  HeapValidate      LIB "KERNEL32.DLL" ALIAS "HeapValidate"     (BYVAL hHeap AS DWORD, BYVAL dwFlags AS DWORD, BYVAL lpMem AS DWORD) AS LONG
                                      
  DECLARE FUNCTION  GetProcessHeap    LIB "KERNEL32.DLL" ALIAS "GetProcessHeap"   () AS LONG
  DECLARE FUNCTION  GetProcessHeaps   LIB "KERNEL32.DLL" ALIAS "GetProcessHeaps"  (BYVAL NumberOfHeaps AS LONG, ProcessHeaps AS LONG) AS LONG

  DECLARE SUB       MoveMemory        LIB "KERNEL32.DLL" ALIAS "RtlMoveMemory"    (pDestination AS ANY, pSource AS ANY, BYVAL cbLength AS LONG)

#ENDIF
  
  
  '----------------------------------------------------------------------------
  ' Allocates a block of nSize memory from a heap.
  ' If the function succeeds, the return value is a pointer to the allocated memory block.
  '----------------------------------------------------------------------------
  FUNCTION da_MemAlloc( BYVAL nSize AS DWORD ) AS LONG

    IF nSize = 0??? THEN
      EXIT FUNCTION
    END IF
    
    FUNCTION = HeapAlloc( GetProcessHeap(), %HEAP_ZERO_MEMORY, MAX( 16, nSize + 1 ) )

  END FUNCTION

  '----------------------------------------------------------------------------
  ' Allocates a block of memory from a heap.
  ' The size of the block is the size of the passed string
  ' The passed string is copied into the new memory block
  ' If the function succeeds, the return value is a pointer to the allocated memory block.
  '----------------------------------------------------------------------------
  FUNCTION da_MemAllocByVal( BYVAL sData AS STRING ) AS LONG
    DIM pData AS DWORD
    
    pData = da_MemAlloc( LEN( sData ) )
    IF pData > 0??? THEN
      POKE$ pData, sData
    END IF
    
    FUNCTION = pData

  END FUNCTION

  '----------------------------------------------------------------------------
  ' Frees a memory block allocated by the HeapAlloc or HeapReAlloc function
  '----------------------------------------------------------------------------
  FUNCTION da_MemFree( BYVAL pMem AS LONG ) AS LONG
 
    IF pMem = 0& THEN
      EXIT FUNCTION
    END IF
    
    IF HeapSize( GetProcessHeap(), 0, BYVAL pMem ) > 0 THEN
      HeapFree GetProcessHeap(), 0, BYVAL pMem
      FUNCTION = 1
    END IF
    
  END FUNCTION

  '----------------------------------------------------------------------------
  ' Retrieves the size of a memory block allocated by the HeapAlloc or HeapReAlloc functions
  '----------------------------------------------------------------------------
  FUNCTION da_MemSize( BYVAL pMem AS LONG ) AS DWORD
    DIM nSize   AS DWORD

    IF pMem = 0& THEN
      EXIT FUNCTION
    END IF
    
    nSize = HeapSize( GetProcessHeap(), 0, BYVAL pMem )
    IF nSize = 0??? OR nSize = %MAXDWORD THEN
      EXIT FUNCTION
    END IF
    FUNCTION = nSize
    
  END FUNCTION

  '----------------------------------------------------------------------------
  ' Allocates or reallocate a block of memory from a heap.
  ' If the function succeeds, the return value is a pointer to the allocated memory block.
  '----------------------------------------------------------------------------
  FUNCTION da_MemReAlloc( BYVAL pMem AS LONG, BYVAL nNewSize AS DWORD ) AS LONG
    DIM nSize   AS DWORD
    DIM pNewMem AS LONG
    
    IF nNewSize = 0??? THEN
      EXIT FUNCTION
    END IF
    
    IF pMem THEN
      nSize = da_MemSize( pMem )
      IF nSize = 0??? THEN EXIT FUNCTION
      pNewMem = da_MemAlloc( nNewSize )
      MoveMemory BYVAL pNewMem, BYVAL pMem, MIN( nNewSize, nSize )
      da_MemFree pMem
    ELSE
      pNewMem = da_MemAlloc( nNewSize )
    END IF
    FUNCTION = pNewMem
    
  END FUNCTION
