#If 0
  =============================================================================
  File NAME   : DynArrays_Test.BAS
  Author      : Eros Olmi
  Version     : 1.0.6
  Description : Dynamic Arrays using strings test program
  Compiler    : PBWIN80
  Web site    : http://www.thinbasic.com
  =============================================================================
  COPYRIGHT AND PERMISSION NOTICE
  =============================================================================
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
  USE OR OTHER DEALINGS IN THE SOFTWARE.
  =============================================================================
  Description:
  -----------------------------------------------------------------------------

  =============================================================================
#EndIf


  #COMPILE EXE
  #INCLUDE "WIN32API.INC"
  #INCLUDE "DynArrays_Mem.inc"
  #INCLUDE "DynArrays.inc"

  %TX_S  = 110
  %BT_S  = 120
  GLOBAL hDlg AS LONG


  SUB OutString(txtMsg AS STRING, OPTIONAL BYVAL ForceRedraw AS LONG)
    DIM RedrawControl AS LONG
    DIM tmpStr AS STRING     
    
    RedrawControl = %FALSE
    IF UCASE$(LEFT$(txtMsg, 8)) = "[MSGBOX]" THEN
      MSGBOX RIGHT$(txtMsg, LEN(txtMsg) - 8)
    ELSE
      'If single char passed, repeat it for 80, ie lines/separators
      IF LEN(txtMsg) = 1 THEN
        txtMsg = REPEAT$(80, txtMsg)         
        RedrawControl = %TRUE       
      END IF
      'If no string passed, than blank the text area
      IF LEN(txtMsg) > 0 THEN CONTROL GET TEXT hDlg, %TX_S TO tmpStr
  
      tmpStr = tmpStr + txtMsg
      IF LEN(txtMsg) > 0 THEN tmpStr = tmpStr + $CRLF
      CONTROL SET TEXT hDlg, %TX_S, tmpStr 
    END IF
    IF RedrawControl = %TRUE OR ForceRedraw = %TRUE THEN
      CONTROL redraw hDlg, %TX_S
    END IF
  END SUB

  FUNCTION DynArrayTest AS LONG
  
    LOCAL Count     AS LONG
  
    LOCAL T0        AS DOUBLE
    LOCAL T1        AS DOUBLE

    LOCAL MaxX      AS LONG
    LOCAL Maxy      AS LONG
    LOCAL MaxZ      AS LONG
    LOCAL x         AS LONG
    LOCAL y         AS LONG
    LOCAL z         AS LONG
    
    LOCAL pArray    AS LONG

    STATIC TestNumber AS LONG
   
    INCR TestNumber
        
    OutString "" 
    OutString "-" 
    OutString "Array test number " + FORMAT$(TestNumber) + " started."
    OutString "-" 

    MaxX = 3
    MaxY = 3
    MaxZ = 2

    OutString "MaxX = " + FORMAT$(MaxX) + "   MaxY = " + FORMAT$(MaxY) + "   MaxZ = " + FORMAT$(MaxZ)
    OutString "for a total of " + FORMAT$(IIF&(MaxX>0&,MaxX,1&)*IIF&(MaxY>0&,MaxY,1&)*IIF&(MaxZ>0&,MaxZ,1&)) + " items."
    OutString "-"
    
    '---
    ' Create a new array with MaxItems of EXT type
    ' Every array can have and ID and a description
    ' More fields can be added to array description area. See INC file
    '---
    T0 = TIMER
    pArray = Array_Dim("TESTNUM", "Test numeric items", %ArrayType_Ext, MaxX, MaxY, MaxZ)
    T1 = TIMER

    OutString "Created a numeric array at " & FORMAT$(pArray) & " of for a total of " &  FORMAT$(Array_GetElementsCount(pArray)) & " items"
    OutString _
            "Array ID                : " + Array_GetUserID(pArray)                      + $CRLF + _
            "Array name              : " + Array_GetName(pArray)                        + $CRLF + _
            "Number of items         : " + FORMAT$(Array_GetElementsCount(pArray))      + $CRLF + _
            "Array total size (bytes): " + FORMAT$(Array_GetSize(pArray))               + $CRLF + _
            "Time to create          : " + FORMAT$(T1 - T0, "#0.000000") + " secs"      + $CRLF + _
            "... now I will fill the Array. It can takes some time ..."                 + $CRLF + _
            "", %TRUE
    DIALOG DOEVENTS
    
    '---
    ' Fill all the items
    '---
    T0 = TIMER
    FOR x = 1 TO MaxX                      
      FOR Y = 1 TO MaxY
        FOR Z = 1 TO MaxZ
          Count = Array_LinearPos(pArray, X, Y, Z)
          Array_SetItem_Num(pArray, Count, X, Y, Z)
          OutString _
                  "    X=" + FORMAT$(X,"000") + _
                  "    Y=" + FORMAT$(Y,"000") + _
                  "    Z=" + FORMAT$(Z,"000") + _
                  "  Pos=" + FORMAT$(Array_LinearPos(pArray, X, Y, Z),"0000") + _
                  "  Val=" + FORMAT$(Array_GetItem_Num(pArray, X, Y, Z),"0000.000")
        NEXT
      NEXT
    NEXT   
    T1 = TIMER

    OutString _
            "Time to fill            : " + FORMAT$(T1 - T0, "#0.000000") + " secs"              + $CRLF + _
            "First element value     : " + FORMAT$(Array_GetItem_Num(pArray, 1&, 1&, 1&))       + $CRLF + _
            "Last element value      : " + FORMAT$(Array_GetItem_Num(pArray, MaxX, MaxY, MaxZ)) + $CRLF + _
            "", %TRUE
    
    OutString "Now redim the array adding some more items."
    DIALOG DOEVENTS
    
    T0 = TIMER

    Array_ReDim(pArray, %FALSE, MaxX, MaxY, MaxZ * 2)
    '---Fill the entire array with a random number
    RANDOMIZE
    CALL Array_Fill_Num(pArray, RND(1, 10000))

    T1 = TIMER
    OutString "New array ia allocated at " & FORMAT$(pArray) & " and has " &  FORMAT$(Array_GetElementsCount(pArray)) & " items."
    OutString _
            "Array ID                : " + Array_GetUserID(pArray)                              + $CRLF + _
            "Array name              : " + Array_GetName(pArray)                                + $CRLF + _
            "Number of items         : " + FORMAT$(Array_GetElementsCount(pArray))              + $CRLF + _
            "Array total size (bytes): " + FORMAT$(Array_GetSize(pArray))                       + $CRLF + _
            "Time to ReDim and fill  : " + FORMAT$(T1 - T0, "#0.000000") + " secs"              + $CRLF + _
            "First element value     : " + FORMAT$(Array_GetItem_Num(pArray, 1&, 1&, 1&))       + $CRLF + _
            "Old Last element value  : " + FORMAT$(Array_GetItem_Num(pArray, MaxX, MaxY, MaxZ)) + $CRLF + _
            "", %TRUE

   
    '---
    ' Transform multidemensional array positions to linear positions
    '---
    OutString "Every coord converted to linear position"
    FOR x = 1 TO MaxX
      FOR Y = 1 TO MaxY
        FOR Z = 1 TO MaxZ * 2
          OutString _
                  "    X=" + FORMAT$(X,"000") + _
                  "    Y=" + FORMAT$(Y,"000") + _
                  "    Z=" + FORMAT$(Z,"000") + _
                  "  Pos=" + FORMAT$(Array_LinearPos(pArray, X, Y, Z),"0000") + _
                  "  Val=" + FORMAT$(Array_GetItem_Num(pArray, X, Y, Z),"0000")
        NEXT  
      NEXT
    NEXT      


    
    '---
    ' One of the problem of this approch is that programmer is responsible
    ' of memory allocation/deallocation. Here we have to deallocate the
    ' memory allocated by the Array constructor
    '---
    OutString "Now erase the EXT array"
    T0 = TIMER
    Array_Erase(pArray)
    T1 = TIMER
    OutString _
            "Time to delete the array: " + FORMAT$(T1 - T0, "#0.000000") + " secs"         + $CRLF


    '---
    ' Now test a dyn string array
    '---
    OutString "-" 
    OutString "Now create a string array"
    OutString "-" 

    pArray = Array_Dim("", "Test STRING items", %ArrayType_String, MaxX, MaxY, MaxZ)
    OutString "OK. " + FORMAT$(Array_GetSize(pArray)) + " bytes needed for " + FORMAT$(Array_GetElementsCount(pArray)) + " string elements."

    '---
    ' Set first element
    '---
    Array_SetItem_Str(pArray, "ABC", 1, 1, 1)                    
    
    '---
    ' Now set last element with some data
    '---
    Array_SetItem_Str(pArray, REPEAT$(10000,"1234567890"), MaxX, MaxY, MaxZ)
    
    OutString _
            "Item number 1 is = " + Array_GetItem_Str(pArray, 1, 1, 1)                  + $CRLF + _
            "Item number " + FORMAT$(Array_GetElementsCount(pArray)) + " is "                   + _
            FORMAT$(LEN(Array_GetItem_Str(pArray, MaxX, MaxY, MaxZ))) + " bytes long."  + $CRLF + _
            ""

    '---
    ' To delete a dynamic string array Array_Erase is needed in order to deallocate
    ' all strings dynamically allocated. In this case PB will not automatically
    ' release the memory. Programmer has to do.
    '---
    OutString "Now erase the String array"
    T0 = TIMER
    Array_Erase(pArray)
    T1 = TIMER
    OutString _
            "Time to delete the array: " + FORMAT$(T1 - T0, "#0.000000") + " secs"      + $CRLF
    


    DIM Items AS LONG
    DIM Counter AS LONG
    
    Items = 1000000
    OutString "-" 
    OutString "Now create a bigger string array of " & FORMAT$(Items) & " items"
    OutString "-" 

    T0 = TIMER
    pArray = Array_Dim("", "Test STRING items", %ArrayType_String, Items)
    T1 = TIMER
    OutString "Time to create the array: " + FORMAT$(T1 - T0, "#0.000000") + " secs"

    T0 = TIMER
    for Counter = 1& to Items
      Array_SetItem_Str(pArray, "WHAT A WORLDWHAT A WORLDWHAT A WORLDWHAT A WORLDWHAT A WORLDWHAT A WORLDWHAT A WORLD"+STR$(Counter), Counter)
    NEXT
    t1 = TIMER
    OutString "Time to fill the array: " + FORMAT$(T1 - T0, "#0.000000") + " secs"

    T0 = TIMER
    Array_Erase(pArray)
    T1 = TIMER
    OutString "Time to delete the array: " + FORMAT$(T1 - T0, "#0.000000") + " secs"

    
  END FUNCTION

    
  '----------------------------------------------------------------------------
  ' Dialog call back
  '----------------------------------------------------------------------------
  CALLBACK FUNCTION DlgProc()
    
    SELECT CASE CBMSG
      CASE %WM_INITDIALOG
        CONTROL ADD TEXTBOX,  hDlg, %TX_S,  "",                               _
                                            1, 1, 0, 0 ,                  _
                                            %ES_WANTRETURN OR %ES_NOHIDESEL	  _
                                            OR %ES_MULTILINE OR %WS_VSCROLL,  _
                                            %WS_EX_STATICEDGE	
      
        CONTROL SET COLOR     hDlg, %TX_S,  %GREEN, %BLACK
      
        CONTROL ADD BUTTON,   hDlg, %BT_S,  "Start Test",                     _
                                            0, 0, 100, 14,                _
                                            %WS_TABSTOP OR %BS_FLAT
      
      CASE %WM_COMMAND
        IF CBCTLMSG = %BN_CLICKED THEN
          IF CBCTL = %BT_S THEN 
            CONTROL SET TEXT hDlg, %BT_S, "wait ..."
            CALL DynArrayTest
            CONTROL SET TEXT hDlg, %BT_S, "ReDo Test"
          END IF
        END IF 
        
      CASE %WM_SIZE   
        LOCAL dX AS LONG
        LOCAL dY AS LONG             
        LOCAL ctX AS LONG
        LOCAL ctY AS LONG             
        
        DIALOG  GET SIZE CBHNDL         TO  dX,  dY
        CONTROL GET SIZE CBHNDL, %TX_S  TO ctX, ctY 

        CONTROL SET SIZE CBHNDL, %TX_S, dX -   6, dY - 41
        CONTROL SET LOC  CBHNDL, %BT_S, dX - 115, dY - 35
          
    END SELECT
    
  END FUNCTION
  
  '----------------------------------------------------------------------------
  ' Main program
  '----------------------------------------------------------------------------
  FUNCTION PBMAIN() AS LONG
  
    LOCAL Result AS LONG
    
    DIALOG FONT "Courier New", 10
  
    DIALOG NEW 0, "DynArray Test",      , ,                               _
                                        335, 310                          _
                                        , %WS_OVERLAPPEDWINDOW	, _
                                        ,                                 _
                                        TO hDlg                           
                                        
    DIALOG SHOW MODAL hDlg CALL DlgProc TO Result
  
  END FUNCTION
